package eel6788;

import java.awt.Toolkit;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import yaes.framework.Version;
import yaes.framework.simulation.Simulation;
import yaes.framework.simulation.SimulationInput;
import yaes.framework.simulation.SimulationOutput;
import yaes.ui.simulationcontrol.SimulationControlGui;
import yaes.ui.text.TextUi;

public class Main implements Constants {
	private static final int TRAINING_SEED = 100; // this is cheating...
	private static final int TEST_SEED = 100;
	private static SimulationInput model = null;
	private static final String MENU_SIMPLE_RUN = "Simple Run";
	private static final String MENU_SIMPLE_RUN_VISUAL =
			"Simple Run with Visual Controller";
	private static final File outputDir = new File("data/directeddiffusion");

	/**
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		TextUi.println(Version.versionString());
		final SimulationInput model = Main.createDefaultSimulationInput();
		if (!Main.outputDir.isDirectory()) {
			TextUi.errorPrint("Output directory: " + Main.outputDir
					+ " does not exist!");
			if (!Main.outputDir.mkdirs()) {
				TextUi.errorPrint("Creation of output directory: "
						+ Main.outputDir + " unsuccessful.");
				return;
			}
		}
		final List<String> menu = new ArrayList<String>();
		menu.add(Main.MENU_SIMPLE_RUN);
		menu.add(Main.MENU_SIMPLE_RUN_VISUAL);
		String result = TextUi.menu(menu, MENU_SIMPLE_RUN, "Choose:");
		// final String result = Main.MENU_SIMPLE_RUN_VISUAL;
		if (result.equals(Main.MENU_SIMPLE_RUN)) {
			Main.doSimpleRun(model, false);
		}
		if (result.equals(Main.MENU_SIMPLE_RUN_VISUAL)) {
			Main.doSimpleRun(model, true);
		}
		Toolkit.getDefaultToolkit().beep();
		TextUi.println("Done, exiting");
		System.exit(0);
	}

	/**
	 * Creates the default simulation input
	 * 
	 * @return
	 */
	private static SimulationInput createDefaultSimulationInput() {
		final int scalingFactor = 1;
		Main.model = new SimulationInput();
		Main.model.setStopTime(5000);
		// model.setParameter(TRANSMISSION_RANGE, 300);
		Main.model.setParameter(Constants.SENSOR_RANGE, 50);
		Main.model.setParameter(Constants.TRANSMISSION_RANGE, 50);
		Main.model.setParameter(Constants.SENSOR_NODES, 64);
		Main.model.setParameter(Constants.ACTUATOR_NODES, 2);
		Main.model.setParameter(Constants.XMAX, 800);
		Main.model.setParameter(Constants.YMAX, 800);
		Main.model.setParameter(Constants.XSINK, 450);
		Main.model.setParameter(Constants.YSINK, 450);
		Main.model.setParameter(SensorArrangement.GRID); // GRID
		// model.setParameter(ActuatorMovement.RANDOM_WAYPOINT); // WINDING
		Main.model.setParameter(ActuatorMovement.ACCROSS); // WINDING
		Main.model.setParameter(VisualDisplay.NO);
		Main.model.setParameter(Constants.VO_MESSAGE_INSPECTOR, 0);
		Main.model.setParameter(Constants.OUTPUTDIR, Main.outputDir.toString());
		return Main.model;
	}

	/**
	 * Does a simple run in the visual mode - for experimentation etc.
	 * 
	 * @param model
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public static void doSimpleRun(SimulationInput model, boolean visual)
			throws InstantiationException, IllegalAccessException {
		final SimulationInput sip = Main.createDefaultSimulationInput();
		sip.setParameter(Constants.SensorType.DIRECTED_DIFFUSION);
		sip.setParameter(VisualDisplay.YES);
		SimulationOutput sop = null;
		final DirectedDiffusionContext context = new DirectedDiffusionContext();
		if (visual) {
			sop =
					SimulationControlGui.simulate(sip,
							DirectedDiffusionSimulation.class, context);
		} else {
			sop =
					Simulation.simulate(sip, DirectedDiffusionSimulation.class,
							context);
		}
		System.out.println(sop);
	}
}
